/*
 * client.c: data structure for SA-clients
 */

/*
 * Spectrum Analyzer plugin for VDR (C++)
 *
 * (C) 2006 Christian Leuschen <christian.leuschen@gmx.de>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#ifndef ___CLIENT_C
#define ___CLIENT_C

#include "client.h"

// cClientBarHeights ##############################################################
cClientBarHeights::cClientBarHeights(int sBands)
{
	bands = sBands;
	barHeightsOld = new int[bands];
	reset();
}
// ----------------------------------------------------------------------------

cClientBarHeights::~cClientBarHeights()
{
	delete [] barHeightsOld;
}
// ----------------------------------------------------------------------------

void cClientBarHeights::reset(void)
{
	int i;
	
	for (i = 0; i < bands; i++)
	{
	  	barHeightsOld[i]	= 0;
	}
}

// cClientData ################################################################
cClientData::cClientData(int sBands)
{
	leftChannel = new cClientBarHeights(sBands);
	rightChannel = new cClientBarHeights(sBands);
}

// ----------------------------------------------------------------------------
cClientData::~cClientData()
{
	delete leftChannel;
	delete rightChannel;
}

// cSaClient ##################################################################
cSaClient::cSaClient(const char *sName, int sBands, int sFalloff)
{
	name = sName ? strdup(sName):0;
	bands = sBands;
	falloff = sFalloff;
	
	clientData = new cClientData(sBands);
	time.Set(DATA_TIMEOUT);
}

cSaClient::~cSaClient()
{
	delete clientData;
	free(name);
}
// ----------------------------------------------------------------------------

int cSaClient::Compare(const cListObject &ListObject) const
{
  cSaClient *obj=(cSaClient *)&ListObject;
  return strcasecmp(name,obj->name);
}

int cSaClient::Compare(const char *sName)
{
  return strcasecmp(sName,this->name);
}

// ----------------------------------------------------------------------------
/*
void cSaClient::reset(void)
{
	clientData->leftChannel->reset();
	clientData->rightChannel->reset();
	time.Set(DATA_TIMEOUT);
}
*/
// ----------------------------------------------------------------------------

void cSaClient::getBarHeightsOfSingleChannel( int *barHeightsTmp,
										int *barHeightsTmpOld,
										double *spectrumDataTmp, int *sVolumeChannel )
{
	int steps = NUM_BANDS/(2*bands);
/*
 * TODO: Check this for 16 and 20 bands resp. (=lcdproc-client)
 * (Taken from xmms-apectrum-analyzer plugin)
 * Given the frequency spectrum for 256 values, divide these 
 * 256 values among the ranges mentioned in scaleXY below and 
 * select the largest frequency among those falling in one of
 * the scale ranges as the height of the "bar" representing
 * that range.
 */
 	// "completely arbitrary" steps
//				  = {0,	1,	2,	3,	5,	7,	10,	14,	20,	28,		40,		54,		74,	101, 137, 187, 255};
//	int steps16[] = {0,	10,	20,	30,	40,	50,	65,	80,	95,	110,	130, 	150,	180,210, 230, 240, 255};
//	int steps20[] = {0, 1, 2, 3, 4, 5, 6, 7, 10, 13, 16, 20, 28, 40, 54, 74, 101, 137, 187, 255};

	int i,j;
	int sum;
	
	j=0;	
	sum = 0;
	*sVolumeChannel = 0;

	if ( bands > NUM_BANDS ) // more bands than processed here are not supported
		return;				 // (should be caught before calling this method)
	else
	{
		for ( i=0; i<NUM_BANDS && j<bands; i++)
		{
			sum += ((int)spectrumDataTmp[i]);

/*			if ( !SpanSetup.PureData && bands == 16 )
			{ // optimized for 16 bands
				steps = steps16[j+1];
			}
			else if ( !SpanSetup.PureData && bands == 20 )
			{ // optimized for 20 bands
				steps = steps20[j+1];
			}
*/
			if ( (i+1) % (int)(steps) == 0 )
			{ // generic computation
				
				barHeightsTmp[j] = (int)(sum/steps);
				
				// Best practice - please let me know if there is a better way
				// to "equalize" the visualization			
				if (SpanSetup.UseLogarithmic)	// it's a try to reduce the "dynamic range" of various frequencies
				{								// e.g. lower frequencies are prevailing (very tall bars "on the left").
					if (barHeightsTmp[j] > 0)
					{
						barHeightsTmp[j] = (int)(log((double)barHeightsTmp[j]*barHeightsTmp[j]*barHeightsTmp[j])*log((double)(i*i+50)));
					}
					else
						barHeightsTmp[j] = 0;
				}
				else if ( !SpanSetup.PureData )	// don't equalize the data in any way
				{
					barHeightsTmp[j] = (int)(barHeightsTmp[j]*log((double)(i))*log((double)(i)));
					barHeightsTmp[j] >>= 3;
				}
				
				// "Cutoff to 100" (HEIGHT == 100), not exact but it works
				if (barHeightsTmp[j] > HEIGHT)
					barHeightsTmp[j] = HEIGHT;
				
				// compute the bar-falloff depending on the height of the last run
				if(barHeightsTmpOld[j] - barHeightsTmp[j] > falloff && falloff != 0)
				{
					barHeightsTmp[j] = barHeightsTmpOld[j] - falloff;
				}
				
				// just to be sure
//				if (barHeightsTmp[j] < 0)
//				{
//					barHeightsTmp[j] = 0;
//				}
				
				// store the height for the next run
				barHeightsTmpOld[j] = barHeightsTmp[j];
				*sVolumeChannel += barHeightsTmp[j];

				j++;
				sum = 0;
			}
		}
		*sVolumeChannel /= bands;
	}
	time.Set(DATA_TIMEOUT);
}
// ----------------------------------------------------------------------------

void cSaClient::getBarHeightsOfRightChannel( int *barHeightsTmp, double *spectrumDataTmp,
						int *sVolumeChannel )
{
/*	if ( time.TimedOut() )
	{   // after DATA_TIMEOUT mseconds without request for visualisation data
		reset();
	}
*/	
	getBarHeightsOfSingleChannel( barHeightsTmp, clientData->rightChannel->barHeightsOld,
					 				spectrumDataTmp, sVolumeChannel );
}
// ----------------------------------------------------------------------------

void cSaClient::getBarHeightsOfLeftChannel( int *barHeightsTmp, double *spectrumDataTmp,
						int *sVolumeChannel )
{
	getBarHeightsOfSingleChannel( barHeightsTmp, clientData->leftChannel->barHeightsOld,
					 				spectrumDataTmp, sVolumeChannel );
	
/*	if ( time.TimedOut() )
	{   // after DATA_TIMEOUT mseconds without request for visualisation data
		reset();
	}
*/
}
// ----------------------------------------------------------------------------

void cSaClient::getBarHeightsOfBothChannels( int *dataBothChannels, int *dataLeftChannel,
										int *dataRightChannel, int *sVolumeLeftChannel,
										int *sVolumeRightChannel, int *sVolumeBothChannels,
										double *spectrumDataLeftChannel, double *spectrumDataRightChannel
						)
{
	int i;

	getBarHeightsOfRightChannel( dataRightChannel, spectrumDataRightChannel, sVolumeRightChannel );
	getBarHeightsOfLeftChannel( dataLeftChannel, spectrumDataLeftChannel, sVolumeLeftChannel );
	
	for (i=0; i<bands; i++)
	{
		dataBothChannels[i] = (int)((dataRightChannel[i] + dataLeftChannel[i])/2);
	}
	*sVolumeBothChannels = (int)(*sVolumeRightChannel + *sVolumeLeftChannel)/2;
}

#endif //___CLIENT_C
