/*
 * spectrum_analyzer.c: The maths behind it
 */

/*
 * Spectrum Analyzer plugin for VDR (C++)
 *
 * (C) 2006 Christian Leuschen <christian.leuschen@gmx.de>
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#ifndef ___SPECTRUM_ANALYZER_C
#define ___SPECTRUM_ANALYZER_C

#include <fftw3.h>

#include "spectrum_analyzer.h"
#include "setup.h"

// cAudioChannel ##############################################################
cAudioChannel::cAudioChannel()
{
	out = (fftw_complex*) fftw_malloc(sizeof(fftw_complex) * FFTW_SIZE);
	plan = fftw_plan_dft_r2c_1d(NUM_BANDS, in, out, FFTW_PATIENT);
	
	cAudioChannel::reset();
}
// ----------------------------------------------------------------------------

cAudioChannel::~cAudioChannel()
{
	fftw_destroy_plan(plan);
	fftw_free(out);
}
// ----------------------------------------------------------------------------

void cAudioChannel::reset(void)
{
	int i;
	
	for (i = 0; i < FFTW_SIZE; i++)
	{
	  	in[i] = 0;
	  	out[i/2][0] = 0;
	  	out[i/2][1] = 0;
	}
	
	for (i = 0; i < NUM_BANDS; i++)
	{
	  	spectrumData[i] = 0;
	}
}
// ----------------------------------------------------------------------------

/*
 * Copy the PCM data to the FFTW input array.
 */ 
void cAudioChannel::initInput(int *data, int length)
{
    int i;
	
	for (i = 0; i < length/2 && i < FFTW_SIZE; i++)
	{
		in[i] = (double)(data[i]);
	}
}
// ----------------------------------------------------------------------------

/*
 * Calculate the FFTW output freq amplitudes from the
 * real and imaginary parts of out[] array.
 */ 
void cAudioChannel::initOutput ()
{
    int i;
    double rl, im;

    for (i = 0; i < NUM_BANDS; i++)
	{
        rl = out[i][0]/FFTW_SIZE;
        im = out[i][1]/FFTW_SIZE;
		spectrumData[i] = (sqrt((rl * rl) + (im * im)));
    }
}
// ----------------------------------------------------------------------------

/*
 * The channel 0 of the stereo PCM data is used to calculate
 * the spectrum using FFTW. FFT/DFT on data sample of size N
 * produces a spectrum of size N/2 since the frequencies are
 * mirror images beyond N/2 (but the phases are opposite).
 */ 
void cAudioChannel::renderPcm(int* data, int length)
{
	reset();
    initInput(data, length);
    fftw_execute(plan);       
    initOutput();
}
// ----------------------------------------------------------------------------

// cAudioChannel ##############################################################
cSpectrumAnalyzer::cSpectrumAnalyzer()
{
	right = new cAudioChannel;
	left = new cAudioChannel;
}
// ----------------------------------------------------------------------------

cSpectrumAnalyzer::~cSpectrumAnalyzer()
{
	delete right;
	delete left;
}
// ----------------------------------------------------------------------------

/*
 * The channel 0 of the stereo PCM data is used to calculate
 * the spectrum using FFTW. FFT/DFT on data sample of size N(=length)
 * produces a spectrum of size N/2 since the frequencies are
 * mirror images beyond N/2 (but the phases are opposite).
 */ 
void cSpectrumAnalyzer::saRenderPcm(int* data, int length)
{
	right->renderPcm( data, length );
	left->renderPcm( data+length/2, length );
	time.Set(DATA_TIMEOUT);
}
// ----------------------------------------------------------------------------

void cSpectrumAnalyzer::getAllData( int bands, int *dataBothChannels, int *dataLeftChannel,
						int *dataRightChannel, int *sVolumeLeftChannel,
						int *sVolumeRightChannel, int *sVolumeBothChannels,
						cSaClient *client )
{
	client->getBarHeightsOfBothChannels( dataBothChannels,
									dataLeftChannel,
									dataRightChannel,
									sVolumeLeftChannel,
									sVolumeRightChannel,
									sVolumeBothChannels,
									left->spectrumData,
									right->spectrumData
								);
}

void cSpectrumAnalyzer::Replaying( const cControl *DvbPlayerControl, const char *Name, const char *FileName, bool On )
{
	if (On && left && right)
	{
		time.Set(DATA_TIMEOUT);
	}
	if (!On && left && right)
	{
		int deletedClients;
		deletedClients = deleteClients();
		
		if ( deletedClients > 0)
		{
			left->reset();
			right->reset();
		}
	}
}

int cSpectrumAnalyzer::deleteClients( void )
{
	int i = 0;
	while ( clients.Count() != 0 )
	{
		clients.Del( clients.Get(0), true );
		i++;
	}
	if ( i>0 )
	{
		dsyslog("[span]: deleted clients: %d (%s()(%s,%d))", i, __func__, __FILE__, __LINE__ );
	}
	return i;
}

#endif //___SPECTRUM_ANALYZER_C
